@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -82,11 +82,10 @@ class SSHAgentKeyPair(SSHKeyPair):
 
     _key_type = 'agent'
 
-    def __init__(self, agent, algorithm, key_blob, comment):
-        super().__init__(algorithm, key_blob, comment)
+    def __init__(self, agent, algorithm, public_data, comment):
+        super().__init__(algorithm, public_data, comment)
 
         self._agent = agent
-        self.key_blob = key_blob
         self.sig_algorithm = algorithm
         self.sig_algorithms = (algorithm,)
         self.host_key_algorithms = self.sig_algorithms
@@ -121,7 +120,7 @@ class SSHAgentKeyPair(SSHKeyPair):
     def set_certificate(self, cert):
         """Set certificate to use with this key"""
 
-        if self.key_blob != cert.key.public_data:
+        if self.public_data != cert.key.public_data:
             raise ValueError('Certificate key mismatch')
 
         self._cert = True
@@ -145,7 +144,7 @@ class SSHAgentKeyPair(SSHKeyPair):
     async def sign(self, data):
         """Sign a block of data with this private key"""
 
-        return await self._agent.sign(self.key_blob, data, self._flags)
+        return await self._agent.sign(self.public_data, data, self._flags)
 
     async def remove(self):
         """Remove this key pair from the agent"""
@@ -225,12 +224,16 @@ class SSHAgentClient:
                 await self._cleanup()
                 raise ValueError(str(exc)) from None
 
-    async def get_keys(self):
+    async def get_keys(self, identities=()):
         """Request the available client keys
 
            This method is a coroutine which returns a list of client keys
            available in the ssh-agent.
 
+           :param identities: (optional)
+               A list of allowed byte string identities to return. If empty,
+               all identities on the SSH agent will be returned.
+
            :returns: A list of :class:`SSHKeyPair` objects
 
         """
@@ -246,6 +249,9 @@ class SSHAgentClient:
                 key_blob = resp.get_string()
                 comment = resp.get_string()
 
+                if identities and key_blob not in identities:
+                    continue
+
                 packet = SSHPacket(key_blob)
                 algorithm = packet.get_string()
 
