@@ -1,11 +1,19 @@
-# Copyright (c) 2013-2015 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -207,10 +215,9 @@ class _Null:
     """A null value"""
 
     @staticmethod
-    def encode(value):
+    def encode(_value):
         """Encode a DER null value"""
 
-        # pylint: disable=unused-argument
         return b''
 
     @classmethod
@@ -334,9 +341,7 @@ class _Sequence:
 
         value = []
         while offset < length:
-            # pylint: disable=unpacking-non-sequence
             item, consumed = der_decode(content[offset:], partial_ok=True)
-            # pylint: enable=unpacking-non-sequence
             value.append(item)
             offset += consumed
 
@@ -365,9 +370,7 @@ class _Set:
 
         value = set()
         while offset < length:
-            # pylint: disable=unpacking-non-sequence
             item, consumed = der_decode(content[offset:], partial_ok=True)
-            # pylint: enable=unpacking-non-sequence
             value.add(item)
             offset += consumed
 
