@@ -1,24 +1,30 @@
-# Copyright (c) 2013-2018 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
 
 """SSH authentication handlers"""
 
-import asyncio
-
-from .constants import DEFAULT_LANG, DISC_PROTOCOL_ERROR
+from .constants import DEFAULT_LANG
 
 from .gss import GSSError
 
-from .misc import DisconnectError, PasswordChangeRequired, get_symbol_names
+from .misc import ProtocolError, PasswordChangeRequired, get_symbol_names
 
 from .packet import Boolean, String, UInt32, SSHPacketHandler
 
@@ -93,8 +99,7 @@ class _ClientAuth(_Auth):
 
         super().__init__(conn, self._start())
 
-    @asyncio.coroutine
-    def _start(self):
+    async def _start(self):
         """Abstract method for starting client authentication"""
 
         # Provided by subclass
@@ -106,35 +111,31 @@ class _ClientAuth(_Auth):
     def auth_failed(self):
         """Callback when auth fails"""
 
-    @asyncio.coroutine
-    def send_request(self, *args, key=None):
+    async def send_request(self, *args, key=None):
         """Send a user authentication request"""
 
-        yield from self._conn.send_userauth_request(self._method,
-                                                    *args, key=key)
+        await self._conn.send_userauth_request(self._method, *args, key=key)
 
 
 class _ClientNullAuth(_ClientAuth):
     """Client side implementation of null auth"""
 
-    @asyncio.coroutine
-    def _start(self):
+    async def _start(self):
         """Start client null authentication"""
 
-        yield from self.send_request()
+        await self.send_request()
 
 
 class _ClientGSSKexAuth(_ClientAuth):
     """Client side implementation of GSS key exchange auth"""
 
-    @asyncio.coroutine
-    def _start(self):
+    async def _start(self):
         """Start client GSS key exchange authentication"""
 
         if self._conn.gss_kex_auth_requested():
             self.logger.debug1('Trying GSS key exchange auth')
 
-            yield from self.send_request(key=self._conn.get_gss_context())
+            await self.send_request(key=self._conn.get_gss_context())
         else:
             self._conn.try_next_auth()
 
@@ -150,16 +151,16 @@ class _ClientGSSMICAuth(_ClientAuth):
         self._gss = None
         self._got_error = False
 
-    @asyncio.coroutine
-    def _start(self):
+    async def _start(self):
         """Start client GSS MIC authentication"""
 
         if self._conn.gss_mic_auth_requested():
             self.logger.debug1('Trying GSS MIC auth')
 
             self._gss = self._conn.get_gss_context()
+            self._gss.reset()
             mechs = b''.join((String(mech) for mech in self._gss.mechs))
-            yield from self.send_request(UInt32(len(self._gss.mechs)), mechs)
+            await self.send_request(UInt32(len(self._gss.mechs)), mechs)
         else:
             self._conn.try_next_auth()
 
@@ -174,16 +175,14 @@ class _ClientGSSMICAuth(_ClientAuth):
         else:
             self.send_packet(MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE)
 
-    def _process_response(self, pkttype, pktid, packet):
+    def _process_response(self, _pkttype, _pktid, packet):
         """Process a GSS response from the server"""
 
-        # pylint: disable=unused-argument
-
         mech = packet.get_string()
         packet.check_end()
 
         if mech not in self._gss.mechs:
-            raise DisconnectError(DISC_PROTOCOL_ERROR, 'Mechanism mismatch')
+            raise ProtocolError('Mechanism mismatch')
 
         try:
             token = self._gss.step()
@@ -200,11 +199,9 @@ class _ClientGSSMICAuth(_ClientAuth):
 
         return True
 
-    def _process_token(self, pkttype, pktid, packet):
+    def _process_token(self, _pkttype, _pktid, packet):
         """Process a GSS token from the server"""
 
-        # pylint: disable=unused-argument
-
         token = packet.get_string()
         packet.check_end()
 
@@ -224,27 +221,23 @@ class _ClientGSSMICAuth(_ClientAuth):
 
         return True
 
-    def _process_error(self, pkttype, pktid, packet):
+    def _process_error(self, _pkttype, _pktid, packet):
         """Process a GSS error from the server"""
 
-        # pylint: disable=unused-argument
-
         _ = packet.get_uint32()         # major_status
         _ = packet.get_uint32()         # minor_status
         msg = packet.get_string()
         _ = packet.get_string()         # lang
         packet.check_end()
 
-        self.logger.warning('GSS error from server: %s', msg)
+        self.logger.debug1('GSS error from server: %s', msg)
         self._got_error = True
 
         return True
 
-    def _process_error_token(self, pkttype, pktid, packet):
+    def _process_error_token(self, _pkttype, _pktid, packet):
         """Process a GSS error token from the server"""
 
-        # pylint: disable=no-self-use,unused-argument
-
         token = packet.get_string()
         packet.check_end()
 
@@ -252,7 +245,7 @@ class _ClientGSSMICAuth(_ClientAuth):
             self._gss.step(token)
         except GSSError as exc:
             if not self._got_error: # pragma: no cover
-                self.logger.warning('GSS error from server: %s', str(exc))
+                self.logger.debug1('GSS error from server: %s', str(exc))
 
         return True
 
@@ -264,16 +257,42 @@ class _ClientGSSMICAuth(_ClientAuth):
     }
 
 
+class _ClientHostBasedAuth(_ClientAuth):
+    """Client side implementation of host based auth"""
+
+    async def _start(self):
+        """Start client host based authentication"""
+
+        keypair, client_host, client_username = \
+            await self._conn.host_based_auth_requested()
+
+        if keypair is None:
+            self._conn.try_next_auth()
+            return
+
+        self.logger.debug1('Trying host based auth of user %s on host %s '
+                           'with %s host key', client_username, client_host,
+                           keypair.algorithm)
+
+        try:
+            await self.send_request(String(keypair.algorithm),
+                                    String(keypair.public_data),
+                                    String(client_host),
+                                    String(client_username), key=keypair)
+        except ValueError as exc:
+            self.logger.debug1('Host based auth failed: %s', exc)
+            self._conn.try_next_auth()
+
+
 class _ClientPublicKeyAuth(_ClientAuth):
     """Client side implementation of public key auth"""
 
     _handler_names = get_symbol_names(globals(), 'MSG_USERAUTH_PK_')
 
-    @asyncio.coroutine
-    def _start(self):
+    async def _start(self):
         """Start client public key authentication"""
 
-        self._keypair = yield from self._conn.public_key_auth_requested()
+        self._keypair = await self._conn.public_key_auth_requested()
 
         if self._keypair is None:
             self._conn.try_next_auth()
@@ -282,34 +301,31 @@ class _ClientPublicKeyAuth(_ClientAuth):
         self.logger.debug1('Trying public key auth with %s key',
                            self._keypair.algorithm)
 
-        yield from self.send_request(Boolean(False),
-                                     String(self._keypair.algorithm),
-                                     String(self._keypair.public_data))
+        await self.send_request(Boolean(False),
+                                String(self._keypair.algorithm),
+                                String(self._keypair.public_data))
 
-    @asyncio.coroutine
-    def _send_signed_request(self):
+    async def _send_signed_request(self):
         """Send signed public key request"""
 
         self.logger.debug1('Signing request with %s key',
                            self._keypair.algorithm)
 
-        yield from self.send_request(Boolean(True),
-                                     String(self._keypair.algorithm),
-                                     String(self._keypair.public_data),
-                                     key=self._keypair)
+        await self.send_request(Boolean(True),
+                                String(self._keypair.algorithm),
+                                String(self._keypair.public_data),
+                                key=self._keypair)
 
-    def _process_public_key_ok(self, pkttype, pktid, packet):
+    def _process_public_key_ok(self, _pkttype, _pktid, packet):
         """Process a public key ok response"""
 
-        # pylint: disable=unused-argument
-
         algorithm = packet.get_string()
         key_data = packet.get_string()
         packet.check_end()
 
         if (algorithm != self._keypair.algorithm or
                 key_data != self._keypair.public_data):
-            raise DisconnectError(DISC_PROTOCOL_ERROR, 'Key mismatch')
+            raise ProtocolError('Key mismatch')
 
         self.create_task(self._send_signed_request())
         return True
@@ -324,11 +340,10 @@ class _ClientKbdIntAuth(_ClientAuth):
 
     _handler_names = get_symbol_names(globals(), 'MSG_USERAUTH_INFO_')
 
-    @asyncio.coroutine
-    def _start(self):
+    async def _start(self):
         """Start client keyboard interactive authentication"""
 
-        submethods = yield from self._conn.kbdint_auth_requested()
+        submethods = await self._conn.kbdint_auth_requested()
 
         if submethods is None:
             self._conn.try_next_auth()
@@ -336,15 +351,14 @@ class _ClientKbdIntAuth(_ClientAuth):
 
         self.logger.debug1('Trying keyboard-interactive auth')
 
-        yield from self.send_request(String(''), String(submethods))
+        await self.send_request(String(''), String(submethods))
 
-    @asyncio.coroutine
-    def _receive_challenge(self, name, instruction, lang, prompts):
+    async def _receive_challenge(self, name, instruction, lang, prompts):
         """Receive and respond to a keyboard interactive challenge"""
 
         responses = \
-            yield from self._conn.kbdint_challenge_received(name, instruction,
-                                                            lang, prompts)
+            await self._conn.kbdint_challenge_received(name, instruction,
+                                                       lang, prompts)
 
         if responses is None:
             self._conn.try_next_auth()
@@ -353,11 +367,9 @@ class _ClientKbdIntAuth(_ClientAuth):
         self.send_packet(MSG_USERAUTH_INFO_RESPONSE, UInt32(len(responses)),
                          b''.join(String(r) for r in responses))
 
-    def _process_info_request(self, pkttype, pktid, packet):
+    def _process_info_request(self, _pkttype, _pktid, packet):
         """Process a keyboard interactive authentication request"""
 
-        # pylint: disable=unused-argument
-
         name = packet.get_string()
         instruction = packet.get_string()
         lang = packet.get_string()
@@ -367,8 +379,8 @@ class _ClientKbdIntAuth(_ClientAuth):
             instruction = instruction.decode('utf-8')
             lang = lang.decode('ascii')
         except UnicodeDecodeError:
-            raise DisconnectError(DISC_PROTOCOL_ERROR, 'Invalid keyboard '
-                                  'interactive info request') from None
+            raise ProtocolError('Invalid keyboard interactive '
+                                'info request') from None
 
         num_prompts = packet.get_uint32()
         prompts = []
@@ -379,8 +391,8 @@ class _ClientKbdIntAuth(_ClientAuth):
             try:
                 prompt = prompt.decode('utf-8')
             except UnicodeDecodeError:
-                raise DisconnectError(DISC_PROTOCOL_ERROR, 'Invalid keyboard '
-                                      'interactive info request') from None
+                raise ProtocolError('Invalid keyboard interactive '
+                                    'info request') from None
 
             prompts.append((prompt, echo))
 
@@ -404,11 +416,10 @@ class _ClientPasswordAuth(_ClientAuth):
 
         self._password_change = False
 
-    @asyncio.coroutine
-    def _start(self):
+    async def _start(self):
         """Start client password authentication"""
 
-        password = yield from self._conn.password_auth_requested()
+        password = await self._conn.password_auth_requested()
 
         if password is None:
             self._conn.try_next_auth()
@@ -416,13 +427,12 @@ class _ClientPasswordAuth(_ClientAuth):
 
         self.logger.debug1('Trying password auth')
 
-        yield from self.send_request(Boolean(False), String(password))
+        await self.send_request(Boolean(False), String(password))
 
-    @asyncio.coroutine
-    def _change_password(self, prompt, lang):
+    async def _change_password(self, prompt, lang):
         """Start password change"""
 
-        result = yield from self._conn.password_change_requested(prompt, lang)
+        result = await self._conn.password_change_requested(prompt, lang)
 
         if result == NotImplemented:
             # Password change not supported - move on to the next auth method
@@ -435,9 +445,9 @@ class _ClientPasswordAuth(_ClientAuth):
 
         self._password_change = True
 
-        yield from self.send_request(Boolean(True),
-                                     String(old_password.encode('utf-8')),
-                                     String(new_password.encode('utf-8')))
+        await self.send_request(Boolean(True),
+                                String(old_password.encode('utf-8')),
+                                String(new_password.encode('utf-8')))
 
     def auth_succeeded(self):
         if self._password_change:
@@ -449,11 +459,9 @@ class _ClientPasswordAuth(_ClientAuth):
             self._password_change = False
             self._conn.password_change_failed()
 
-    def _process_password_change(self, pkttype, pktid, packet):
+    def _process_password_change(self, _pkttype, _pktid, packet):
         """Process a password change request"""
 
-        # pylint: disable=unused-argument
-
         prompt = packet.get_string()
         lang = packet.get_string()
 
@@ -461,8 +469,7 @@ class _ClientPasswordAuth(_ClientAuth):
             prompt = prompt.decode('utf-8')
             lang = lang.decode('ascii')
         except UnicodeDecodeError:
-            raise DisconnectError(DISC_PROTOCOL_ERROR,
-                                  'Invalid password change request') from None
+            raise ProtocolError('Invalid password change request') from None
 
         self.auth_failed()
         self.create_task(self._change_password(prompt, lang))
@@ -483,8 +490,7 @@ class _ServerAuth(_Auth):
 
         super().__init__(conn, self._start(packet))
 
-    @asyncio.coroutine
-    def _start(self, packet):
+    async def _start(self, packet):
         """Abstract method for starting server authentication"""
 
         # Provided by subclass
@@ -505,14 +511,12 @@ class _ServerNullAuth(_ServerAuth):
     """Server side implementation of null auth"""
 
     @classmethod
-    def supported(cls, conn):
+    def supported(cls, _conn):
         """Return that null authentication is never a supported auth mode"""
 
-        # pylint: disable=unused-argument
         return False
 
-    @asyncio.coroutine
-    def _start(self, packet):
+    async def _start(self, packet):
         """Supported always returns false, so we never get here"""
 
 
@@ -530,8 +534,7 @@ class _ServerGSSKexAuth(_ServerAuth):
 
         return conn.gss_kex_auth_supported()
 
-    @asyncio.coroutine
-    def _start(self, packet):
+    async def _start(self, packet):
         """Start server GSS key exchange authentication"""
 
         mic = packet.get_string()
@@ -542,9 +545,9 @@ class _ServerGSSKexAuth(_ServerAuth):
         data = self._conn.get_userauth_request_data(self._method)
 
         if (self._gss.complete and self._gss.verify(data, mic) and
-                (yield from self._conn.validate_gss_principal(self._username,
-                                                              self._gss.user,
-                                                              self._gss.host))):
+                (await self._conn.validate_gss_principal(self._username,
+                                                         self._gss.user,
+                                                         self._gss.host))):
             self.send_success()
         else:
             self.send_failure()
@@ -566,8 +569,7 @@ class _ServerGSSMICAuth(_ServerAuth):
 
         return conn.gss_mic_auth_supported()
 
-    @asyncio.coroutine
-    def _start(self, packet):
+    async def _start(self, packet):
         """Start server GSS MIC authentication"""
 
         mechs = set()
@@ -592,22 +594,19 @@ class _ServerGSSMICAuth(_ServerAuth):
 
         self.send_packet(MSG_USERAUTH_GSSAPI_RESPONSE, String(match))
 
-    @asyncio.coroutine
-    def _finish(self):
+    async def _finish(self):
         """Finish server GSS MIC authentication"""
 
-        if (yield from self._conn.validate_gss_principal(self._username,
-                                                         self._gss.user,
-                                                         self._gss.host)):
+        if (await self._conn.validate_gss_principal(self._username,
+                                                    self._gss.user,
+                                                    self._gss.host)):
             self.send_success()
         else:
             self.send_failure()
 
-    def _process_token(self, pkttype, pktid, packet):
+    def _process_token(self, _pkttype, _pktid, packet):
         """Process a GSS token from the client"""
 
-        # pylint: disable=unused-argument
-
         token = packet.get_string()
         packet.check_end()
 
@@ -628,11 +627,9 @@ class _ServerGSSMICAuth(_ServerAuth):
 
         return True
 
-    def _process_exchange_complete(self, pkttype, pktid, packet):
+    def _process_exchange_complete(self, _pkttype, _pktid, packet):
         """Process a GSS exchange complete message from the client"""
 
-        # pylint: disable=unused-argument
-
         packet.check_end()
 
         if self._gss.complete and not self._gss.provides_integrity:
@@ -642,26 +639,22 @@ class _ServerGSSMICAuth(_ServerAuth):
 
         return True
 
-    def _process_error_token(self, pkttype, pktid, packet):
+    def _process_error_token(self, _pkttype, _pktid, packet):
         """Process a GSS error token from the client"""
 
-        # pylint: disable=unused-argument
-
         token = packet.get_string()
         packet.check_end()
 
         try:
             self._gss.step(token)
         except GSSError as exc:
-            self.logger.warning('GSS error from client: %s', str(exc))
+            self.logger.debug1('GSS error from client: %s', str(exc))
 
         return True
 
-    def _process_mic(self, pkttype, pktid, packet):
+    def _process_mic(self, _pkttype, _pktid, packet):
         """Process a GSS MIC from the client"""
 
-        # pylint: disable=unused-argument
-
         mic = packet.get_string()
         packet.check_end()
 
@@ -683,6 +676,46 @@ class _ServerGSSMICAuth(_ServerAuth):
     }
 
 
+class _ServerHostBasedAuth(_ServerAuth):
+    """Server side implementation of host based auth"""
+
+    @classmethod
+    def supported(cls, conn):
+        """Return whether host based authentication is supported"""
+
+        return conn.host_based_auth_supported()
+
+    async def _start(self, packet):
+        """Start server host based authentication"""
+
+        algorithm = packet.get_string()
+        key_data = packet.get_string()
+        client_host = packet.get_string()
+        client_username = packet.get_string()
+        msg = packet.get_consumed_payload()
+        signature = packet.get_string()
+
+        packet.check_end()
+
+        try:
+            client_host = client_host.decode('utf-8')
+            client_username = saslprep(client_username.decode('utf-8'))
+        except (UnicodeDecodeError, SASLPrepError):
+            raise ProtocolError('Invalid host-based auth request') from None
+
+        self.logger.debug1('Verifying host based auth of user %s '
+                           'on host %s with %s host key', client_username,
+                           client_host, algorithm)
+
+        if (await self._conn.validate_host_based_auth(self._username,
+                                                      key_data, client_host,
+                                                      client_username,
+                                                      msg, signature)):
+            self.send_success()
+        else:
+            self.send_failure()
+
+
 class _ServerPublicKeyAuth(_ServerAuth):
     """Server side implementation of public key auth"""
 
@@ -692,8 +725,7 @@ class _ServerPublicKeyAuth(_ServerAuth):
 
         return conn.public_key_auth_supported()
 
-    @asyncio.coroutine
-    def _start(self, packet):
+    async def _start(self, packet):
         """Start server public key authentication"""
 
         sig_present = packet.get_boolean()
@@ -714,8 +746,8 @@ class _ServerPublicKeyAuth(_ServerAuth):
         else:
             self.logger.debug1('Trying public key auth with %s key', algorithm)
 
-        if (yield from self._conn.validate_public_key(self._username, key_data,
-                                                      msg, signature)):
+        if (await self._conn.validate_public_key(self._username, key_data,
+                                                 msg, signature)):
             if sig_present:
                 self.send_success()
             else:
@@ -736,8 +768,7 @@ class _ServerKbdIntAuth(_ServerAuth):
 
         return conn.kbdint_auth_supported()
 
-    @asyncio.coroutine
-    def _start(self, packet):
+    async def _start(self, packet):
         """Start server keyboard interactive authentication"""
 
         lang = packet.get_string()
@@ -748,14 +779,13 @@ class _ServerKbdIntAuth(_ServerAuth):
             lang = lang.decode('ascii')
             submethods = submethods.decode('utf-8')
         except UnicodeDecodeError:
-            raise DisconnectError(DISC_PROTOCOL_ERROR, 'Invalid keyboard '
-                                  'interactive auth request') from None
+            raise ProtocolError('Invalid keyboard interactive '
+                                'auth request') from None
 
         self.logger.debug1('Trying keyboard-interactive auth')
 
-        challenge = yield from self._conn.get_kbdint_challenge(self._username,
-                                                               lang,
-                                                               submethods)
+        challenge = await self._conn.get_kbdint_challenge(self._username,
+                                                          lang, submethods)
         self._send_challenge(challenge)
 
     def _send_challenge(self, challenge):
@@ -776,20 +806,16 @@ class _ServerKbdIntAuth(_ServerAuth):
         else:
             self.send_failure()
 
-    @asyncio.coroutine
-    def _validate_response(self, responses):
+    async def _validate_response(self, responses):
         """Validate a keyboard interactive authentication response"""
 
         next_challenge = \
-            yield from self._conn.validate_kbdint_response(self._username,
-                                                           responses)
+            await self._conn.validate_kbdint_response(self._username, responses)
         self._send_challenge(next_challenge)
 
-    def _process_info_response(self, pkttype, pktid, packet):
+    def _process_info_response(self, _pkttype, _pktid, packet):
         """Process a keyboard interactive authentication response"""
 
-        # pylint: disable=unused-argument
-
         num_responses = packet.get_uint32()
         responses = []
         for _ in range(num_responses):
@@ -798,8 +824,8 @@ class _ServerKbdIntAuth(_ServerAuth):
             try:
                 response = response.decode('utf-8')
             except UnicodeDecodeError:
-                raise DisconnectError(DISC_PROTOCOL_ERROR, 'Invalid keyboard '
-                                      'interactive info response') from None
+                raise ProtocolError('Invalid keyboard interactive '
+                                    'info response') from None
 
             responses.append(response)
 
@@ -822,8 +848,7 @@ class _ServerPasswordAuth(_ServerAuth):
 
         return conn.password_auth_supported()
 
-    @asyncio.coroutine
-    def _start(self, packet):
+    async def _start(self, packet):
         """Start server password authentication"""
 
         password_change = packet.get_boolean()
@@ -835,22 +860,20 @@ class _ServerPasswordAuth(_ServerAuth):
             password = saslprep(password.decode('utf-8'))
             new_password = saslprep(new_password.decode('utf-8'))
         except (UnicodeDecodeError, SASLPrepError):
-            raise DisconnectError(DISC_PROTOCOL_ERROR, 'Invalid password auth '
-                                  'request') from None
+            raise ProtocolError('Invalid password auth request') from None
 
         try:
             if password_change:
                 self.logger.debug1('Trying to chsnge password')
 
-                result = yield from self._conn.change_password(self._username,
-                                                               password,
-                                                               new_password)
+                result = await self._conn.change_password(self._username,
+                                                          password,
+                                                          new_password)
             else:
                 self.logger.debug1('Trying password auth')
 
                 result = \
-                    yield from self._conn.validate_password(self._username,
-                                                            password)
+                    await self._conn.validate_password(self._username, password)
 
             if result:
                 self.send_success()
@@ -869,6 +892,13 @@ def register_auth_method(alg, client_handler, server_handler):
     _server_auth_handlers[alg] = server_handler
 
 
+def get_client_auth_methods():
+    """Return a list of supported client auth methods"""
+
+    return [method for method in _client_auth_handlers
+            if method != b'none']
+
+
 def lookup_client_auth(conn, method):
     """Look up the client authentication method to use"""
 
@@ -879,7 +909,7 @@ def lookup_client_auth(conn, method):
 
 
 def get_server_auth_methods(conn):
-    """Return a list of supported auth methods"""
+    """Return a list of supported server auth methods"""
 
     auth_methods = []
 
@@ -908,6 +938,7 @@ _auth_method_list = (
     (b'none',                 _ClientNullAuth,      _ServerNullAuth),
     (b'gssapi-keyex',         _ClientGSSKexAuth,    _ServerGSSKexAuth),
     (b'gssapi-with-mic',      _ClientGSSMICAuth,    _ServerGSSMICAuth),
+    (b'hostbased',            _ClientHostBasedAuth, _ServerHostBasedAuth),
     (b'publickey',            _ClientPublicKeyAuth, _ServerPublicKeyAuth),
     (b'keyboard-interactive', _ClientKbdIntAuth,    _ServerKbdIntAuth),
     (b'password',             _ClientPasswordAuth,  _ServerPasswordAuth)
