@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -291,7 +291,7 @@ class SSHChannel(SSHPacketHandler):
         while self._recv_buf and not self._recv_paused:
             self._deliver_data(*self._recv_buf.pop(0))
 
-        if not self._recv_buf:
+        if not self._recv_buf and self._recv_paused != 'starting':
             if self._encoding and not exc and \
                     self._recv_state in ('eof_pending', 'close_pending'):
                 try:
@@ -1002,9 +1002,10 @@ class SSHClientChannel(SSHChannel):
         self._exit_status = None
         self._exit_signal = None
 
-    async def create(self, session_factory, command, subsystem, env, term_type,
-                     term_size, term_modes, x11_forwarding, x11_display,
-                     x11_auth_path, x11_single_connection, agent_forwarding):
+    async def create(self, session_factory, command, subsystem, env,
+                     request_pty, term_type, term_size, term_modes,
+                     x11_forwarding, x11_display, x11_auth_path,
+                     x11_single_connection, agent_forwarding):
         """Create an SSH client session"""
 
         self.logger.info('Requesting new SSH session')
@@ -1025,8 +1026,8 @@ class SSHClientChannel(SSHChannel):
             self.logger.debug1('  Env: %s=%s', name, value)
             self._send_request(b'env', String(str(name)), String(str(value)))
 
-        if term_type:
-            self.logger.debug1('  Terminal type: %s', term_type)
+        if request_pty:
+            self.logger.debug1('  Terminal type: %s', term_type or 'None')
 
             if not term_size:
                 width = height = pixwidth = pixheight = 0
@@ -1053,7 +1054,8 @@ class SSHClientChannel(SSHChannel):
 
             modes += Byte(PTY_OP_END)
 
-            if not (await self._make_request(b'pty-req', String(term_type),
+            if not (await self._make_request(b'pty-req',
+                                             String(term_type or ''),
                                              UInt32(width), UInt32(height),
                                              UInt32(pixwidth),
                                              UInt32(pixheight),
@@ -1335,7 +1337,7 @@ class SSHServerChannel(SSHChannel):
     _write_datatypes = {EXTENDED_DATA_STDERR}
 
     def __init__(self, conn, loop, allow_pty, line_editor, line_history,
-                 encoding, errors, window, max_pktsize):
+                 max_line_length, encoding, errors, window, max_pktsize):
         """Initialize an SSH server channel"""
 
         super().__init__(conn, loop, encoding, errors, window, max_pktsize)
@@ -1345,6 +1347,7 @@ class SSHServerChannel(SSHChannel):
         self._allow_pty = allow_pty
         self._line_editor = line_editor
         self._line_history = line_history
+        self._max_line_length = max_line_length
         self._term_type = None
         self._term_size = (0, 0, 0, 0)
         self._term_modes = {}
@@ -1356,7 +1359,8 @@ class SSHServerChannel(SSHChannel):
         """Wrap a line editor around the session if enabled"""
 
         if self._line_editor:
-            chan = SSHLineEditorChannel(self, session, self._line_history)
+            chan = SSHLineEditorChannel(self, session, self._line_history,
+                                        self._max_line_length)
             session = SSHLineEditorSession(chan, session)
         else:
             chan = self
