@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2018 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -25,6 +25,9 @@
 # Default language for error messages
 DEFAULT_LANG                        = 'en-US'
 
+# Default SSH listening port
+DEFAULT_PORT                        = 22
+
 # SSH message codes
 MSG_DISCONNECT                      = 1
 MSG_IGNORE                          = 2
