@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -33,6 +33,7 @@ class _DSAKey(SSHKey):
     """Handler for DSA public key encryption"""
 
     algorithm = b'ssh-dss'
+    default_hash_alg = 'sha1'
     pem_name = b'DSA'
     pkcs8_oid = ObjectIdentifier('1.2.840.10040.4.1')
     sig_algorithms = (algorithm,)
@@ -208,7 +209,7 @@ class _DSAKey(SSHKey):
         if not self._key.x:
             raise ValueError('Private key needed for signing')
 
-        r, s = der_decode(self._key.sign(data))
+        r, s = der_decode(self._key.sign(data, 'sha1'))
         return String(r.to_bytes(20, 'big') + s.to_bytes(20, 'big'))
 
     def verify_ssh(self, data, sig_algorithm, packet):
@@ -225,13 +226,13 @@ class _DSAKey(SSHKey):
         r = int.from_bytes(sig[:20], 'big')
         s = int.from_bytes(sig[20:], 'big')
 
-        return self._key.verify(data, der_encode((r, s)))
+        return self._key.verify(data, der_encode((r, s)), 'sha1')
 
 
-register_public_key_alg(b'ssh-dss', _DSAKey)
+register_public_key_alg(b'ssh-dss', _DSAKey, False)
 
 register_certificate_alg(1, b'ssh-dss', b'ssh-dss-cert-v01@openssh.com',
-                         _DSAKey, SSHOpenSSHCertificateV01)
+                         _DSAKey, SSHOpenSSHCertificateV01, False)
 
 for alg in _DSAKey.x509_algorithms:
-    register_x509_certificate_alg(alg)
+    register_x509_certificate_alg(alg, False)
