@@ -1,11 +1,19 @@
-# Copyright (c) 2017 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2017-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -40,16 +48,12 @@ except ImportError: # pragma: no cover
     class GSSClient:
         """Stub client class for reporting that GSS is not available"""
 
-        def __init__(self, host, delegate_creds):
-            # pylint: disable=unused-argument
-
+        def __init__(self, _host, _delegate_creds):
             raise GSSError()
 
 
     class GSSServer:
         """Stub client class for reporting that GSS is not available"""
 
-        def __init__(self, host):
-            # pylint: disable=unused-argument
-
+        def __init__(self, _host):
             raise GSSError()
