@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -31,10 +31,18 @@ from .socks import SSHSOCKSForwarder
 class SSHListener:
     """SSH listener for inbound connections"""
 
+    def __init__(self):
+        self._tunnel = None
+
     async def __aenter__(self):
         return self
 
     async def __aexit__(self, *exc_info):
+        if self._tunnel:
+            self._tunnel.close()
+            await self._tunnel.wait_closed()
+            self._tunnel = None
+
         self.close()
         await self.wait_closed()
 
@@ -55,6 +63,11 @@ class SSHListener:
 
         return 0
 
+    def set_tunnel(self, tunnel):
+        """Set tunnel associated with listener"""
+
+        self._tunnel = tunnel
+
     def close(self):
         """Stop listening for new connections
 
@@ -81,6 +94,8 @@ class SSHClientListener(SSHListener):
 
     def __init__(self, conn, session_factory,
                  encoding, errors, window, max_pktsize):
+        super().__init__()
+
         self._conn = conn
         self._session_factory = session_factory
         self._encoding = encoding
@@ -176,8 +191,12 @@ class SSHUNIXClientListener(SSHClientListener):
 class SSHForwardListener(SSHListener):
     """A listener used when forwarding traffic from local ports"""
 
-    def __init__(self, servers, listen_port=0):
+    def __init__(self, conn, servers, listen_key, listen_port=0):
+        super().__init__()
+
+        self._conn = conn
         self._servers = servers
+        self._listen_key = listen_key
         self._listen_port = listen_port
 
     def get_port(self):
@@ -188,8 +207,13 @@ class SSHForwardListener(SSHListener):
     def close(self):
         """Close this listener"""
 
-        for server in self._servers:
-            server.close()
+        if self._conn:
+            self._conn.close_forward_listener(self._listen_key)
+
+            for server in self._servers:
+                server.close()
+
+            self._conn = None
 
     async def wait_closed(self):
         """Wait for this listener to finish closing"""
@@ -257,7 +281,8 @@ async def create_tcp_local_listener(conn, loop, protocol_factory,
         server = await loop.create_server(protocol_factory, sock=sock)
         servers.append(server)
 
-    return SSHForwardListener(servers, listen_port)
+    listen_key = listen_host, listen_port
+    return SSHForwardListener(conn, servers, listen_key, listen_port)
 
 
 async def create_tcp_forward_listener(conn, loop, coro,
@@ -283,7 +308,7 @@ async def create_unix_forward_listener(conn, loop, coro, listen_path):
 
     server = await loop.create_unix_server(protocol_factory, listen_path)
 
-    return SSHForwardListener([server])
+    return SSHForwardListener(conn, [server], listen_path)
 
 
 async def create_socks_listener(conn, loop, coro, listen_host, listen_port):
