@@ -1,11 +1,19 @@
-# Copyright (c) 2015 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2015-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -36,12 +44,11 @@ class WildcardPattern:
 class WildcardHostPattern(WildcardPattern):
     """Match a host name or address against a wildcard pattern"""
 
-    def matches(self, host, addr, ip):
+    def matches(self, host, addr, _ip):
         """Return whether a host or address matches a wild card host pattern"""
 
         # Arguments vary by class, but inheritance is still needed here
-        # IP matching is only done for CIDRHostPattern
-        # pylint: disable=arguments-differ,unused-argument
+        # pylint: disable=arguments-differ
 
         return (host and super().matches(host)) or \
                (addr and super().matches(addr))
@@ -53,12 +60,9 @@ class CIDRHostPattern:
     def __init__(self, pattern):
         self._network = ip_network(pattern)
 
-    def matches(self, host, addr, ip):
+    def matches(self, _host, _addr, ip):
         """Return whether an IP address matches a CIDR address pattern"""
 
-        # Host & addr matching is only done for WildcardHostPattern
-        # pylint: disable=unused-argument
-
         return ip and ip in self._network
 
 
