@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -29,10 +29,22 @@ from .public_key import register_public_key_alg, register_certificate_alg
 from .public_key import register_x509_certificate_alg
 
 
+# pylint: disable=bad-whitespace
+
+_hash_algs = {b'ssh-rsa':        'sha1',
+              b'rsa-sha2-256':   'sha256',
+              b'rsa-sha2-512':   'sha512',
+              b'rsa1024-sha1':   'sha1',
+              b'rsa2048-sha256': 'sha256'}
+
+# pylint: enable=bad-whitespace
+
+
 class _RSAKey(SSHKey):
     """Handler for RSA public key encryption"""
 
     algorithm = b'ssh-rsa'
+    default_hash_alg = 'sha256'
     pem_name = b'RSA'
     pkcs8_oid = ObjectIdentifier('1.2.840.113549.1.1.1')
     sig_algorithms = (b'rsa-sha2-256', b'rsa-sha2-512', b'ssh-rsa')
@@ -195,7 +207,7 @@ class _RSAKey(SSHKey):
         if not self._key.d:
             raise ValueError('Private key needed for signing')
 
-        return String(self._key.sign(data, sig_algorithm))
+        return String(self._key.sign(data, _hash_algs[sig_algorithm]))
 
     def verify_ssh(self, data, sig_algorithm, packet):
         """Verify an SSH-encoded signature of the specified data"""
@@ -203,23 +215,23 @@ class _RSAKey(SSHKey):
         sig = packet.get_string()
         packet.check_end()
 
-        return self._key.verify(data, sig, sig_algorithm)
+        return self._key.verify(data, sig, _hash_algs[sig_algorithm])
 
     def encrypt(self, data, algorithm):
         """Encrypt a block of data with this key"""
 
-        return self._key.encrypt(data, algorithm)
+        return self._key.encrypt(data, _hash_algs[algorithm])
 
     def decrypt(self, data, algorithm):
         """Decrypt a block of data with this key"""
 
-        return self._key.decrypt(data, algorithm)
+        return self._key.decrypt(data, _hash_algs[algorithm])
 
 
-register_public_key_alg(b'ssh-rsa', _RSAKey)
+register_public_key_alg(b'ssh-rsa', _RSAKey, True)
 
 register_certificate_alg(1, b'ssh-rsa', b'ssh-rsa-cert-v01@openssh.com',
-                         _RSAKey, SSHOpenSSHCertificateV01)
+                         _RSAKey, SSHOpenSSHCertificateV01, True)
 
 for alg in _RSAKey.x509_algorithms:
-    register_x509_certificate_alg(alg)
+    register_x509_certificate_alg(alg, True)
