@@ -1,11 +1,19 @@
-# Copyright (c) 2013-2018 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -16,18 +24,19 @@
 class SSHServer:
     """SSH server protocol handler
 
-       Applications should subclass this when implementing an SSH server.
-       At a minimum, one or more of the authentication handlers will need
-       to be overridden to perform authentication, or :meth:`begin_auth`
-       should be overridden to return `False` to indicate that no
-       authentication is required.
+       Applications may subclass this when implementing an SSH server to
+       provide custom authentication and request handlers.
 
-       In addition, one or more of the :meth:`session_requested`,
+       The method :meth:`begin_auth` can be overridden decide whether
+       or not authentication is required, and additional callbacks are
+       provided for each form of authentication in cases where authentication
+       information is not provided in the call to :func:`create_server`.
+
+       In addition, the methods :meth:`session_requested`,
        :meth:`connection_requested`, :meth:`server_requested`,
        :meth:`unix_connection_requested`, or :meth:`unix_server_requested`
-       methods will need to be overridden to handle requests to open
-       sessions or direct connections or set up listeners for forwarded
-       connections.
+       can be overridden to handle requests to open sessions or direct
+       connections or set up listeners for forwarded connections.
 
        .. note:: The authentication callbacks described here can be
                  defined as coroutines. However, they may be cancelled if
@@ -52,8 +61,6 @@ class SSHServer:
 
         """
 
-        pass # pragma: no cover
-
     def connection_lost(self, exc):
         """Called when a connection is lost or closed
 
@@ -64,8 +71,6 @@ class SSHServer:
 
         """
 
-        pass # pragma: no cover
-
     def debug_msg_received(self, msg, lang, always_display):
         """A debug message was received on this connection
 
@@ -85,8 +90,6 @@ class SSHServer:
 
         """
 
-        pass # pragma: no cover
-
     def begin_auth(self, username):
         """Authentication has been requested by the client
 
@@ -113,13 +116,25 @@ class SSHServer:
 
         return True # pragma: no cover
 
+    def auth_completed(self):
+        """Authentication was completed successfully
+
+           This method is called when authentication has completed
+           succesfully. Applications may use this method to perform
+           processing based on the authenticated username or options in
+           the authorized keys list or certificate associated with the
+           user before any sessions are opened or forwarding requests
+           are handled.
+
+        """
+
     def validate_gss_principal(self, username, user_principal,
                                host_principal):
         """Return whether a GSS principal is valid for this user
 
            This method should return `True` if the specified user
            principal is valid for the user being authenticated. It can
-           be overridden by applications wishing to perform their
+           be overridden by applications wishing to perform their own
            authentication.
 
            If blocking operations need to be performed to determine the
@@ -149,14 +164,164 @@ class SSHServer:
         host_domain = host_principal.rsplit('@')[-1]
         return user_principal == username + '@' + host_domain
 
+    def host_based_auth_supported(self):
+        """Return whether or not host-based authentication is supported
+
+           This method should return `True` if client host-based
+           authentication is supported. Applications wishing to support
+           it must have this method return `True` and implement
+           :meth:`validate_host_public_key` and/or :meth:`validate_host_ca_key`
+           to return whether or not the key provided by the client is valid
+           for the client host being authenticated.
+
+           By default, it returns `False` indicating the client host
+           based authentication is not supported.
+
+           :returns: A `bool` indicating if host-based authentication is
+                     supported or not
+
+        """
+
+        return False # pragma: no cover
+
+    def validate_host_public_key(self, client_host, client_addr,
+                                 client_port, key):
+        """Return whether key is an authorized host key for this client host
+
+           Host key based client authentication can be supported by
+           passing authorized host keys in the `known_client_hosts`
+           argument of :func:`create_server`. However, for more flexibility
+           in matching on the allowed set of keys, this method can be
+           implemented by the application to do the matching itself. It
+           should return `True` if the specified key is a valid host key
+           for the client host being authenticated.
+
+           This method may be called multiple times with different keys
+           provided by the client. Applications should precompute as
+           much as possible in the :meth:`begin_auth` method so that
+           this function can quickly return whether the key provided is
+           in the list.
+
+           By default, this method returns `False` for all client host keys.
+
+               .. note:: This function only needs to report whether the
+                         public key provided is a valid key for this client
+                         host. If it is, AsyncSSH will verify that the
+                         client possesses the corresponding private key
+                         before allowing the authentication to succeed.
+
+           :param client_host:
+               The hostname of the client host
+           :param client_addr:
+               The IP address of the client host
+           :param client_port:
+               The port number on the client host
+           :param key:
+               The host public key sent by the client
+           :type client_host: `str`
+           :type client_addr: `str`
+           :type client_port: `int`
+           :type key: :class:`SSHKey` *public key*
+
+           :returns: A `bool` indicating if the specified key is a valid
+                     key for the client host being authenticated
+
+        """
+
+        return False # pragma: no cover
+
+    def validate_host_ca_key(self, client_host, client_addr,
+                             client_port, key):
+        """Return whether key is an authorized CA key for this client host
+
+           Certificate based client host authentication can be
+           supported by passing authorized host CA keys in the
+           `known_client_hosts` argument of :func:`create_server`.
+           However, for more flexibility in matching on the allowed
+           set of keys, this method can be implemented by the application
+           to do the matching itself. It should return `True` if the
+           specified key is a valid certificate authority key for the
+           client host being authenticated.
+
+           This method may be called multiple times with different keys
+           provided by the client. Applications should precompute as
+           much as possible in the :meth:`begin_auth` method so that
+           this function can quickly return whether the key provided is
+           in the list.
+
+           By default, this method returns `False` for all CA keys.
+
+               .. note:: This function only needs to report whether the
+                         public key provided is a valid CA key for this
+                         client host. If it is, AsyncSSH will verify that
+                         the certificate is valid, that the client host is
+                         one of the valid principals for the certificate,
+                         and that the client possesses the private key
+                         corresponding to the public key in the certificate
+                         before allowing the authentication to succeed.
+
+           :param client_host:
+               The hostname of the client host
+           :param client_addr:
+               The IP address of the client host
+           :param client_port:
+               The port number on the client host
+           :param key:
+               The public key which signed the certificate sent by the client
+           :type client_host: `str`
+           :type client_addr: `str`
+           :type client_port: `int`
+           :type key: :class:`SSHKey` *public key*
+
+           :returns: A `bool` indicating if the specified key is a valid
+                     CA key for the client host being authenticated
+
+        """
+
+        return False # pragma: no cover
+
+    def validate_host_based_user(self, username, client_host, client_username):
+        """Return whether remote host and user is authorized for this user
+
+           This method should return `True` if the specified client host
+           and user is valid for the user being authenticated. It can be
+           overridden by applications wishing to enforce restrictions on
+           which remote users are allowed to authenticate as particular
+           local users.
+
+           If blocking operations need to be performed to determine the
+           validity of the client host and user, this method may be defined
+           as a coroutine.
+
+           By default, this method will return `True` when the client
+           username matches the name of the user being authenticated.
+
+           :param username:
+               The user being authenticated
+           :param client_host:
+               The hostname of the client host making the request
+           :param client_username:
+               The username of the user on the client host
+           :type username: `str`
+           :type client_host: `str`
+           :type client_username: `str`
+
+           :returns: A `bool` indicating if the specified client host
+                     and user is valid for the user being authenticated
+
+        """
+
+        return username == client_username
+
     def public_key_auth_supported(self):
         """Return whether or not public key authentication is supported
 
            This method should return `True` if client public key
            authentication is supported. Applications wishing to support
            it must have this method return `True` and implement
-           :meth:`validate_public_key` to return whether or not the key
-           provided by the client is valid for the user being authenticated.
+           :meth:`validate_public_key` and/or :meth:`validate_ca_key`
+           to return whether or not the key provided by the client is
+           valid for the user being authenticated.
 
            By default, it returns `False` indicating the client public
            key authentication is not supported.
@@ -171,7 +336,7 @@ class SSHServer:
     def validate_public_key(self, username, key):
         """Return whether key is an authorized client key for this user
 
-           Basic key-based client authentication can be supported by
+           Key based client authentication can be supported by
            passing authorized keys in the `authorized_client_keys`
            argument of :func:`create_server`, or by calling
            :meth:`set_authorized_keys
@@ -216,8 +381,8 @@ class SSHServer:
     def validate_ca_key(self, username, key):
         """Return whether key is an authorized CA key for this user
 
-           Basic key-based client authentication can be supported by
-           passing authorized keys in the `authorized_client_keys`
+           Certificate based client authentication can be supported by
+           passing authorized CA keys in the `authorized_client_keys`
            argument of :func:`create_server`, or by calling
            :meth:`set_authorized_keys
            <SSHServerConnection.set_authorized_keys>` on the server
@@ -618,11 +783,10 @@ class SSHServer:
 
            :returns: One of the following:
 
-                     * An :class:`SSHListener` object or a coroutine
-                       which returns an :class:`SSHListener` or `False`
-                       if the listener can't be opened
+                     * An :class:`SSHListener` object
                      * `True` to set up standard port forwarding
                      * `False` to reject the request
+                     * A coroutine object which returns one of the above
 
         """
 
