@@ -1,13 +1,21 @@
-#!/usr/bin/env python3.5
+#!/usr/bin/env python3.6
 #
-# Copyright (c) 2013-2017 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2018 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -24,7 +32,7 @@ passwords = {'guest': '',                 # guest account with no password
 
 def handle_client(process):
     process.stdout.write('Welcome to my SSH server, %s!\n' %
-                         process.channel.get_extra_info('username'))
+                         process.get_extra_info('username'))
     process.exit(0)
 
 class MySSHServer(asyncssh.SSHServer):
