@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -59,6 +59,7 @@ class ServerTestCase(AsyncTestCase):
     _agent_pid = None
 
     @classmethod
+    @async_context_manager
     async def listen(cls, *, server_factory=(), options=None, **kwargs):
         """Create an SSH server for the tests to use"""
 
@@ -73,6 +74,7 @@ class ServerTestCase(AsyncTestCase):
                                      options=options, **kwargs)
 
     @classmethod
+    @async_context_manager
     async def listen_reverse(cls, *, options=None, **kwargs):
         """Create a reverse SSH server for the tests to use"""
 
@@ -129,7 +131,8 @@ class ServerTestCase(AsyncTestCase):
         skey_ecdsa.write_public_key('skey_ecdsa.pub')
 
         skey_cert = skey.generate_host_certificate(skey, 'name',
-                                                   principals=['127.0.0.1'])
+                                                   principals=['127.0.0.1',
+                                                               'localhost'])
         skey_cert.write_certificate('skey-cert.pub')
 
         exp_cert = skey.generate_host_certificate(skey, 'name',
@@ -200,6 +203,7 @@ class ServerTestCase(AsyncTestCase):
         shutil.copy('ckey_ecdsa.pub', os.path.join('.ssh', 'id_ecdsa.pub'))
         shutil.copy('ckey_encrypted', os.path.join('.ssh', 'id_rsa'))
         shutil.copy('ckey.pub', os.path.join('.ssh', 'id_rsa.pub'))
+        shutil.copy('ckey-cert.pub', os.path.join('.ssh', 'id_rsa-cert.pub'))
 
         with open('authorized_keys', 'w') as auth_keys:
             with open('ckey.pub') as ckey_pub:
@@ -229,6 +233,7 @@ class ServerTestCase(AsyncTestCase):
 
         os.environ['LOGNAME'] = 'guest'
         os.environ['HOME'] = '.'
+        os.environ['USERPROFILE'] = '.'
 
         cls._server = await cls.start_server()
 
@@ -244,7 +249,7 @@ class ServerTestCase(AsyncTestCase):
             with open('skey.pub') as skey_pub:
                 shutil.copyfileobj(skey_pub, known_hosts)
 
-            known_hosts.write('@cert-authority ' + host)
+            known_hosts.write('@cert-authority * ')
 
             with open('skey.pub') as skey_pub:
                 shutil.copyfileobj(skey_pub, known_hosts)
