@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -23,6 +23,7 @@
 import asyncio
 from copy import copy
 import os
+import socket
 import sys
 import unittest
 from unittest.mock import patch
@@ -44,7 +45,18 @@ from asyncssh.mac import _HMAC, _mac_handler, get_mac_algs
 from asyncssh.packet import Boolean, NameList, String, UInt32
 
 from .server import Server, ServerTestCase
-from .util import asynctest, gss_available, patch_gss, x509_available
+
+from .util import asynctest, gss_available, patch_gss
+from .util import patch_getnameinfo, x509_available
+
+
+class _CheckAlgsClientConnection(asyncssh.SSHClientConnection):
+    """Test specification of encryption algorithms"""
+
+    def get_enc_algs(self):
+        """Return the selected encryption algorithms"""
+
+        return self._enc_algs
 
 
 class _SplitClientConnection(asyncssh.SSHClientConnection):
@@ -269,8 +281,10 @@ class _CloseDuringAuthServer(Server):
 class _InternalErrorServer(Server):
     """Server for testing internal error during auth"""
 
-    def begin_auth(self, username):
-        """Raise an internal error during auth"""
+    def debug_msg_received(self, msg, lang, always_display):
+        """Process a debug message"""
+
+        # pylint: disable=unused-argument
 
         raise RuntimeError('Exception handler test')
 
@@ -332,6 +346,9 @@ class _TestConnection(ServerTestCase):
             conn.logger.info('Acceptor called')
 
         return (await cls.create_server(_TunnelServer, gss_host=(),
+                                        compression_algs='*',
+                                        encryption_algs='*',
+                                        kex_algs='*', mac_algs='*',
                                         acceptor=acceptor))
 
     async def get_server_host_key(self, **kwargs):
@@ -386,6 +403,15 @@ class _TestConnection(ServerTestCase):
         with self.assertRaises(OSError):
             await asyncssh.connect('0.0.0.1', agent_path=None)
 
+    @asynctest
+    async def test_connect_tcp_keepalive_off(self):
+        """Test connecting with TCP keepalive disabled"""
+
+        async with self.connect(tcp_keepalive=False) as conn:
+            sock = conn.get_extra_info('socket')
+            self.assertEqual(bool(sock.getsockopt(socket.SOL_SOCKET,
+                                                  socket.SO_KEEPALIVE)), False)
+
     @asynctest
     async def test_split_version(self):
         """Test version split across two packets"""
@@ -569,8 +595,16 @@ class _TestConnection(ServerTestCase):
     async def test_read_known_hosts(self):
         """Test connecting with known hosts object from read_known_hosts"""
 
-        known_hosts_path = os.path.join('.ssh', 'known_hosts')
-        known_hosts = asyncssh.read_known_hosts(known_hosts_path)
+        known_hosts = asyncssh.read_known_hosts('~/.ssh/known_hosts')
+
+        async with self.connect(known_hosts=known_hosts):
+            pass
+
+    @asynctest
+    async def test_read_known_hosts_filelist(self):
+        """Test connecting with known hosts from read_known_hosts file list"""
+
+        known_hosts = asyncssh.read_known_hosts(['~/.ssh/known_hosts'])
 
         async with self.connect(known_hosts=known_hosts):
             pass
@@ -679,6 +713,43 @@ class _TestConnection(ServerTestCase):
                 async with self.connect(kex_algs=[kex], gss_host='1'):
                     pass
 
+    @asynctest
+    async def test_duplicate_encryption_algs(self):
+        """Test connecting with an duplicated encryption algorithm"""
+
+        with patch('asyncssh.connection.SSHClientConnection',
+                   _CheckAlgsClientConnection):
+            async with self.connect(
+                    encryption_algs=['aes256-ctr', 'aes256-ctr']) as conn:
+                self.assertEqual(conn.get_enc_algs(), [b'aes256-ctr'])
+
+    @asynctest
+    async def test_leading_encryption_alg(self):
+        """Test adding a new first encryption algorithm"""
+
+        with patch('asyncssh.connection.SSHClientConnection',
+                   _CheckAlgsClientConnection):
+            async with self.connect(encryption_algs='^aes256-ctr') as conn:
+                self.assertEqual(conn.get_enc_algs()[0], b'aes256-ctr')
+
+    @asynctest
+    async def test_trailing_encryption_alg(self):
+        """Test adding a new last encryption algorithm"""
+
+        with patch('asyncssh.connection.SSHClientConnection',
+                   _CheckAlgsClientConnection):
+            async with self.connect(encryption_algs='+3des-cbc') as conn:
+                self.assertEqual(conn.get_enc_algs()[-1], b'3des-cbc')
+
+    @asynctest
+    async def test_removing_encryption_alg(self):
+        """Test removing an encryption algorithm"""
+
+        with patch('asyncssh.connection.SSHClientConnection',
+                   _CheckAlgsClientConnection):
+            async with self.connect(encryption_algs='-aes256-ctr') as conn:
+                self.assertTrue(b'aes256-ctr' not in conn.get_enc_algs())
+
     @asynctest
     async def test_empty_kex_algs(self):
         """Test connecting with an empty list of key exchange algorithms"""
@@ -693,6 +764,13 @@ class _TestConnection(ServerTestCase):
         with self.assertRaises(ValueError):
             await self.connect(kex_algs=['xxx'])
 
+    @asynctest
+    async def test_invalid_kex_alg_str(self):
+        """Test connecting with invalid key exchange algorithm pattern"""
+
+        with self.assertRaises(ValueError):
+            await self.connect(kex_algs='xxx')
+
     @asynctest
     async def test_unsupported_kex_alg(self):
         """Test connecting with unsupported key exchange algorithm"""
@@ -910,6 +988,15 @@ class _TestConnection(ServerTestCase):
             conn = await self.connect(keepalive_interval=0.1)
             await conn.wait_closed()
 
+    @asynctest
+    async def test_client_keepalive_string(self):
+        """Test sending keepalive from client with string argument"""
+
+        with patch('asyncssh.connection.SSHServerConnection',
+                   _KeepaliveServerConnection):
+            conn = await self.connect(keepalive_interval='0.1s')
+            await conn.wait_closed()
+
     @asynctest
     async def test_client_set_keepalive_interval(self):
         """Test sending keepalive interval with set_keepalive"""
@@ -917,9 +1004,24 @@ class _TestConnection(ServerTestCase):
         with patch('asyncssh.connection.SSHServerConnection',
                    _KeepaliveServerConnection):
             conn = await self.connect()
-            conn.set_keepalive(0.1)
+            conn.set_keepalive('0m0.1s')
             await conn.wait_closed()
 
+    @asynctest
+    async def test_invalid_client_keepalive(self):
+        """Test setting invalid keepalive from client"""
+
+        with self.assertRaises(ValueError):
+            await self.connect(keepalive_interval=-1)
+
+    @asynctest
+    async def test_client_set_invalid_keepalive_interval(self):
+        """Test setting invalid keepalive interval with set_keepalive"""
+
+        async with self.connect() as conn:
+            with self.assertRaises(ValueError):
+                conn.set_keepalive(interval=-1)
+
     @asynctest
     async def test_client_set_keepalive_count_max(self):
         """Test sending keepalive count max with set_keepalive"""
@@ -930,6 +1032,21 @@ class _TestConnection(ServerTestCase):
             conn.set_keepalive(count_max=10)
             await conn.wait_closed()
 
+    @asynctest
+    async def test_invalid_client_keepalive_count_max(self):
+        """Test setting invalid keepalive count max from client"""
+
+        with self.assertRaises(ValueError):
+            await self.connect(keepalive_count_max=-1)
+
+    @asynctest
+    async def test_client_set_invalid_keepalive_count_max(self):
+        """Test setting invalid keepalive count max with set_keepalive"""
+
+        async with self.connect() as conn:
+            with self.assertRaises(ValueError):
+                conn.set_keepalive(count_max=-1)
+
     @asynctest
     async def test_client_keepalive_failure(self):
         """Test client keepalive failure"""
@@ -940,14 +1057,70 @@ class _TestConnection(ServerTestCase):
             await conn.wait_closed()
 
     @asynctest
-    async def test_rekey(self):
-        """Test SSH re-keying"""
+    async def test_rekey_bytes(self):
+        """Test SSH re-keying with byte limit"""
 
         async with self.connect(rekey_bytes=1) as conn:
             await asyncio.sleep(0.1)
             conn.send_debug('test')
             await asyncio.sleep(0.1)
 
+    @asynctest
+    async def test_rekey_bytes_string(self):
+        """Test SSH re-keying with string byte limit"""
+
+        async with self.connect(rekey_bytes='1') as conn:
+            await asyncio.sleep(0.1)
+            conn.send_debug('test')
+            await asyncio.sleep(0.1)
+
+    @asynctest
+    async def test_invalid_rekey_bytes(self):
+        """Test invalid rekey bytes"""
+
+        for desc, rekey_bytes in (
+                ('Negative inteeger ', -1),
+                ('Missing value', ''),
+                ('Missing integer', 'k'),
+                ('Invalid integer', '!'),
+                ('Invalid integer', '!'),
+                ('Invalid suffix', '1x')):
+            with self.subTest(desc):
+                with self.assertRaises(ValueError):
+                    await self.connect(rekey_bytes=rekey_bytes)
+
+    @asynctest
+    async def test_rekey_seconds(self):
+        """Test SSH re-keying with time limit"""
+
+        async with self.connect(rekey_seconds=0.1) as conn:
+            await asyncio.sleep(0.1)
+            conn.send_debug('test')
+            await asyncio.sleep(0.1)
+
+    @asynctest
+    async def test_rekey_seconds_string(self):
+        """Test SSH re-keying with string time limit"""
+
+        async with self.connect(rekey_seconds='0m0.1s') as conn:
+            await asyncio.sleep(0.1)
+            conn.send_debug('test')
+            await asyncio.sleep(0.1)
+
+    @asynctest
+    async def test_rekey_time_disabled(self):
+        """Test SSH re-keying by time being disabled"""
+
+        async with self.connect(rekey_seconds=None):
+            pass
+
+    @asynctest
+    async def test_invalid_rekey_seconds(self):
+        """Test invalid rekey seconds"""
+
+        with self.assertRaises(ValueError):
+            await self.connect(rekey_seconds=-1)
+
     @asynctest
     async def test_kex_in_progress(self):
         """Test starting SSH key exchange while it is in progress"""
@@ -1018,6 +1191,13 @@ class _TestConnection(ServerTestCase):
 
         await conn.wait_closed()
 
+    @asynctest
+    async def test_username_too_long(self):
+        """Test username being too long in userauth request"""
+
+        with self.assertRaises(asyncssh.IllegalUserName):
+            await self.connect(username=2048*'a')
+
     @asynctest
     async def test_extra_userauth_request(self):
         """Test userauth request after auth is complete"""
@@ -1184,6 +1364,28 @@ class _TestConnectionAsyncAcceptor(ServerTestCase):
             async with self.connect():
                 pass
 
+@patch_gss
+class _TestConnectionServerCerts(ServerTestCase):
+    """Unit tests for AsyncSSH server using server_certs argument"""
+
+    @classmethod
+    async def start_server(cls):
+        """Start an SSH server to connect to"""
+
+        return (await cls.create_server(_TunnelServer, gss_host=(),
+                                        compression_algs='*',
+                                        encryption_algs='*',
+                                        kex_algs='*', mac_algs='*',
+                                        server_host_keys='skey',
+                                        server_host_certs='skey-cert.pub'))
+
+    @asynctest
+    async def test_connect(self):
+        """Test connecting with async context manager"""
+
+        async with self.connect(known_hosts=([], ['skey.pub'], [])):
+            pass
+
 
 class _TestConnectionReverse(ServerTestCase):
     """Unit test for reverse direction connections"""
@@ -1276,7 +1478,8 @@ class _TestConnectionKeepalive(ServerTestCase):
     async def start_server(cls):
         """Start an SSH server which sends keepalive messages"""
 
-        return await cls.create_server(keepalive_interval=0.1)
+        return await cls.create_server(keepalive_interval=0.1,
+                                       keepalive_count_max=3)
 
     @asynctest
     async def test_server_keepalive(self):
@@ -1576,8 +1779,10 @@ class _TestServerInternalError(ServerTestCase):
     async def test_server_internal_error(self):
         """Test server internal error during auth"""
 
-        with self.assertRaises(asyncssh.ConnectionLost):
-            await self.connect()
+        with self.assertRaises(asyncssh.ChannelOpenError):
+            conn = await self.connect()
+            conn.send_debug('Test')
+            await conn.run()
 
 
 class _TestInvalidAuthBanner(ServerTestCase):
@@ -1697,3 +1902,26 @@ class _TestCustomServerVersion(ServerTestCase):
 
         with self.assertRaises(ValueError):
             await self.create_server(server_version='xxx\0')
+
+
+@patch_getnameinfo
+class _TestReverseDNS(ServerTestCase):
+    """Unit test for reverse DNS lookup of client address"""
+
+    @classmethod
+    async def start_server(cls):
+        """Start an SSH server which sends a custom version"""
+
+        with open('config', 'w') as f:
+            f.write('Match host localhost\nPubkeyAuthentication no')
+
+        return await cls.create_server(
+            authorized_client_keys='authorized_keys', rdns_lookup=True,
+            config='config')
+
+    @asynctest
+    async def test_reverse_dns(self):
+        """Test reverse DNS of the client address"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='ckey')
