@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -27,7 +27,7 @@ import unittest
 from unittest.mock import patch
 
 import asyncssh
-from asyncssh.misc import async_context_manager
+from asyncssh.misc import async_context_manager, write_file
 from asyncssh.packet import String
 from asyncssh.public_key import CERT_TYPE_USER, CERT_TYPE_HOST
 
@@ -361,12 +361,19 @@ class _KbdintServer(Server):
         """Return a password challenge after the instructions"""
 
         if self._kbdint_round == 0:
-            result = ('', '', '', [('Password:', False)])
+            if username == 'pw':
+                result = ('', '', '', [('Password:', False)])
+            elif username == 'pc':
+                result = ('', '', '', [('Passcode:', False)])
+            elif username == 'multi':
+                result = ('', '', '', [('Prompt1:', True), ('Prompt2', True)])
+            else:
+                result = ('', '', '', [('Other Challenge:', False)])
         else:
-            if len(responses) == 1 and responses[0] == 'kbdint':
+            if responses in (['kbdint'], ['1', '2']):
                 result = True
             else:
-                result = ('', '', '', [('Other Challenge:', True)])
+                result = ('', '', '', [('Second Challenge:', True)])
 
         self._kbdint_round += 1
         return result
@@ -414,7 +421,7 @@ class _TestGSSAuth(ServerTestCase):
     async def test_gss_kex_auth(self):
         """Test GSS key exchange authentication"""
 
-        async with self.connect(kex_algs=['gss-gex-sha1'],
+        async with self.connect(kex_algs=['gss-gex-sha256'],
                                 username='user', gss_host='1'):
             pass
 
@@ -426,6 +433,29 @@ class _TestGSSAuth(ServerTestCase):
                                 username='user', gss_host='1'):
             pass
 
+    @asynctest
+    async def test_gss_delegate(self):
+        """Test GSS credential delegation"""
+
+        async with self.connect(username='user', gss_host='1',
+                                gss_delegate_creds=True):
+            pass
+
+    @asynctest
+    async def test_gss_kex_disabled(self):
+        """Test GSS key exchange being disabled"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='user', gss_host=(), gss_kex=False,
+                               preferred_auth='gssapi-keyex')
+
+    @asynctest
+    async def test_gss_auth_disabled(self):
+        """Test GSS authentication being disabled"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='user', gss_host=(), gss_auth=False)
+
     @asynctest
     async def test_gss_auth_unavailable(self):
         """Test GSS authentication being unavailable"""
@@ -441,6 +471,36 @@ class _TestGSSAuth(ServerTestCase):
             await self.connect(gss_host='1,init_error', username='user')
 
 
+@unittest.skipUnless(gss_available, 'GSS not available')
+@patch_gss
+class _TestGSSServerAuthDisabled(ServerTestCase):
+    """Unit tests for with GSS key exchange and auth disabled on server"""
+
+    @classmethod
+    async def start_server(cls):
+        """Start an SSH server with GSS key exchange and auth disabled"""
+
+        return await cls.create_server(gss_host='1', gss_kex=False,
+                                       gss_auth=False)
+
+    @asynctest
+    async def test_gss_kex_unavailable(self):
+        """Test GSS key exchange being unavailable"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='user', gss_host=(),
+                               preferred_auth='gssapi-keyex')
+
+    @asynctest
+    async def test_gss_auth_unavailable(self):
+        """Test GSS authentication being unavailable"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='user', gss_host=(),
+                               preferred_auth='gssapi-with-mic')
+
+
+
 @unittest.skipUnless(gss_available, 'GSS not available')
 @patch_gss
 class _TestGSSServerError(ServerTestCase):
@@ -510,6 +570,14 @@ class _TestHostBasedAuth(ServerTestCase):
                                 client_username='user'):
             pass
 
+    @asynctest
+    async def test_client_host_auth_disabled(self):
+        """Test connecting with host-based authentication disabled"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='user', client_host_keys='skey',
+                               client_username='user', host_based_auth=False)
+
     @asynctest
     async def test_client_host_key_bytes(self):
         """Test client host key passed in as bytes"""
@@ -880,6 +948,14 @@ class _TestPublicKeyAuth(ServerTestCase):
 
         return conn
 
+    @asynctest
+    async def test_client_certs(self):
+        """Test trusted client certificate via client_certs"""
+
+        async with self.connect(username='ckey', client_keys='ckey',
+                                client_certs='ckey-cert.pub'):
+            pass
+
     @asynctest
     async def test_agent_auth(self):
         """Test connecting with ssh-agent authentication"""
@@ -890,6 +966,53 @@ class _TestPublicKeyAuth(ServerTestCase):
         async with self.connect(username='ckey'):
             pass
 
+    @asynctest
+    async def test_agent_identities(self):
+        """Test connecting with ssh-agent auth with specific identities"""
+
+        if not self.agent_available(): # pragma: no cover
+            self.skipTest('ssh-agent not available')
+
+        ckey = asyncssh.read_private_key('ckey')
+        ckey.write_private_key('ckey.pem', 'pkcs8-pem')
+
+        ckey_cert = asyncssh.read_certificate('ckey-cert.pub')
+        ckey_ecdsa = asyncssh.read_public_key('ckey_ecdsa.pub')
+
+        for pubkey in ('ckey-cert.pub', 'ckey_ecdsa.pub', 'ckey.pem',
+                       ckey_cert, ckey_ecdsa, ckey_ecdsa.public_data):
+            async with self.connect(username='ckey', agent_identities=pubkey):
+                pass
+
+    @asynctest
+    async def test_agent_identities_config(self):
+        """Test connecting with ssh-agent auth and IdentitiesOnly config"""
+
+        if not self.agent_available(): # pragma: no cover
+            self.skipTest('ssh-agent not available')
+
+        write_file('ckey_err', b'')
+
+        write_file('config', 'IdentitiesOnly True\n'
+                   'IdentityFile ckey-cert.pub\n'
+                   'IdentityFile ckey_ecdsa.pub\n'
+                   'IdentityFile ckey_err\n', 'w')
+
+        async with self.connect(username='ckey', config='config'):
+            pass
+
+    @asynctest
+    async def test_agent_identities_config_default_keys(self):
+        """Test connecting with ssh-agent auth and default IdentitiesOnly"""
+
+        if not self.agent_available(): # pragma: no cover
+            self.skipTest('ssh-agent not available')
+
+        write_file('config', 'IdentitiesOnly True\n', 'w')
+
+        async with self.connect(username='ckey', config='config'):
+            pass
+
     @asynctest
     async def test_agent_signature_algs(self):
         """Test ssh-agent keys with specific signature algorithms"""
@@ -929,6 +1052,22 @@ class _TestPublicKeyAuth(ServerTestCase):
         async with self.connect(username='ckey', client_keys='ckey'):
             pass
 
+    @asynctest
+    async def test_public_key_auth_disabled(self):
+        """Test connecting with public key authentication disabled"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='ckey', client_keys='ckey',
+                               public_key_auth=False)
+
+    @asynctest
+    async def test_public_key_auth_not_preferred(self):
+        """Test public key authentication not being in preferred auth list"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='ckey', client_keys='ckey',
+                               preferred_auth='password')
+
     @asynctest
     async def test_public_key_signature_algs(self):
         """Test public key authentication with specific signature algorithms"""
@@ -1020,7 +1159,8 @@ class _TestPublicKeyAuth(ServerTestCase):
         """Test untrusted client key"""
 
         with self.assertRaises(asyncssh.PermissionDenied):
-            await self.connect(username='ckey', client_keys='skey')
+            await self.connect(username='ckey', client_keys='skey',
+                               agent_path=None)
 
     @asynctest
     async def test_missing_cert(self):
@@ -1041,7 +1181,8 @@ class _TestPublicKeyAuth(ServerTestCase):
                                 valid_before=1)
 
         with self.assertRaises(asyncssh.PermissionDenied):
-            await self.connect(username='ckey', client_keys=[(skey, cert)])
+            await self.connect(username='ckey', client_keys=[(skey, cert)],
+                               agent_path=None)
 
     @asynctest
     async def test_allowed_address(self):
@@ -1070,7 +1211,8 @@ class _TestPublicKeyAuth(ServerTestCase):
                                 options={'source-address': String('0.0.0.0')})
 
         with self.assertRaises(asyncssh.PermissionDenied):
-            await self.connect(username='ckey', client_keys=[(skey, cert)])
+            await self.connect(username='ckey', client_keys=[(skey, cert)],
+                               agent_path=None)
 
     @asynctest
     async def test_untrusted_ca(self):
@@ -1082,7 +1224,8 @@ class _TestPublicKeyAuth(ServerTestCase):
                                 CERT_TYPE_USER, skey, skey, ['skey'])
 
         with self.assertRaises(asyncssh.PermissionDenied):
-            await self.connect(username='ckey', client_keys=[(skey, cert)])
+            await self.connect(username='ckey', client_keys=[(skey, cert)],
+                               agent_path=None)
 
     @asynctest
     async def test_mismatched_ca(self):
@@ -1134,22 +1277,6 @@ class _TestPublicKeyAuth(ServerTestCase):
                                     agent_path=None):
                 pass
 
-    @asynctest
-    async def test_auth_options_reuse(self):
-        """Test public key auth via SSHClientConnectionOptions"""
-
-        async def connect():
-            """Connect to the server using options"""
-
-            async with asyncssh.connect(self._server_addr, self._server_port,
-                                        options=options):
-                pass
-
-        options = asyncssh.SSHClientConnectionOptions(
-            username='ckey', client_keys=[('ckey', None)], gss_host=None)
-
-        await asyncio.gather(connect(), connect())
-
 
 class _TestPublicKeyAsyncServerAuth(_TestPublicKeyAuth):
     """Unit tests for public key authentication with async server callbacks"""
@@ -1246,7 +1373,30 @@ class _TestPreloadedAuthorizedKeys(ServerTestCase):
 
     @asynctest
     async def test_pre_loaded_authorized_keys(self):
-        """Test set_authorized_keys with pre-loaded authorized keys"""
+        """Test pre-loaded authorized keys file"""
+
+        async with self.connect(username='ckey', client_keys='ckey'):
+            pass
+
+
+class _TestPreloadedAuthorizedKeysFileList(ServerTestCase):
+    """Unit tests with pre-loaded authorized keys file list"""
+
+    @classmethod
+    async def start_server(cls):
+        """Start an SSH server which supports public key authentication"""
+
+        def server_factory():
+            """Return an SSH server which calls set_authorized_keys"""
+
+            authorized_keys = asyncssh.read_authorized_keys(['authorized_keys'])
+            return _PublicKeyServer(authorized_keys=authorized_keys)
+
+        return await cls.create_server(server_factory)
+
+    @asynctest
+    async def test_pre_loaded_authorized_keys(self):
+        """Test pre-loaded authorized keys file list"""
 
         async with self.connect(username='ckey', client_keys='ckey'):
             pass
@@ -1417,6 +1567,14 @@ class _TestPasswordAuth(ServerTestCase):
         async with self.connect(username='pw', password='pw', client_keys=None):
             pass
 
+    @asynctest
+    async def test_password_auth_disabled(self):
+        """Test connecting with password authentication disabled"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='pw', password='kbdint',
+                               password_auth=False, preferred_auth='password')
+
     @asynctest
     async def test_password_auth_failure(self):
         """Test _failure connecting with password authentication"""
@@ -1491,13 +1649,45 @@ class _TestKbdintAuth(ServerTestCase):
         return conn
 
     @asynctest
-    async def test_kbdint_auth(self):
-        """Test connecting with keyboard-interactive authentication"""
+    async def test_kbdint_auth_password(self):
+        """Test keyboard-interactive authentication via password"""
 
-        async with self.connect(username='kbdint', password='kbdint',
+        async with self.connect(username='pw', password='kbdint',
                                 client_keys=None):
             pass
 
+    @asynctest
+    async def test_kbdint_auth_passcode(self):
+        """Test keyboard-interactive authentication via passcode"""
+
+        async with self.connect(username='pc', password='kbdint',
+                                client_keys=None):
+            pass
+
+    @asynctest
+    async def test_kbdint_auth_not_password(self):
+        """Test keyboard-interactive authentication other than password"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='kbdint', password='kbdint',
+                               client_keys=None)
+
+    @asynctest
+    async def test_kbdint_auth_multi_not_password(self):
+        """Test keyboard-interactive authentication with multiple prompts"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='multi', password='kbdint',
+                               client_keys=None)
+
+    @asynctest
+    async def test_kbdint_auth_disabled(self):
+        """Test connecting with keyboard-interactive authentication disabled"""
+
+        with self.assertRaises(asyncssh.PermissionDenied):
+            await self.connect(username='pw', password='kbdint',
+                               kbdint_auth=False)
+
     @asynctest
     async def test_kbdint_auth_failure(self):
         """Test failure connecting with keyboard-interactive authentication"""
@@ -1513,9 +1703,16 @@ class _TestKbdintAuth(ServerTestCase):
         async with self._connect_kbdint('kbdint', ['kbdint'], test_async=True):
             pass
 
+    @asynctest
+    async def test_kbdint_auth_callback_multi(self):
+        """Test keyboard-interactive auth callback with multiple challenges"""
+
+        async with self._connect_kbdint('multi', ['1', '2'], test_async=True):
+            pass
+
     @asynctest
     async def test_kbdint_auth_callback_faliure(self):
-        """Test failure connection with keyboard-interactive auth callback"""
+        """Test failure connecting with keyboard-interactive auth callback"""
 
         with self.assertRaises(asyncssh.PermissionDenied):
             await self._connect_kbdint('kbdint', ['badpw'])
@@ -1577,7 +1774,7 @@ class _TestKbdintPasswordServerAuth(ServerTestCase):
             await self._connect_kbdint('pw', ['oldpw'])
 
 
-class _TestClientLoginTimeoutExceeded(ServerTestCase):
+class _TestClientLoginTimeout(ServerTestCase):
     """Unit test for client login timeout"""
 
     @classmethod
@@ -1600,6 +1797,21 @@ class _TestClientLoginTimeoutExceeded(ServerTestCase):
             await self.connect(username='ckey', client_keys='ckey',
                                login_timeout=1)
 
+    @asynctest
+    async def test_client_login_timeout_exceeded_string(self):
+        """Test client login timeout exceeded with string value"""
+
+        with self.assertRaises(asyncssh.ConnectionLost):
+            await self.connect(username='ckey', client_keys='ckey',
+                               login_timeout='0m1s')
+
+    @asynctest
+    async def test_invalid_client_login_timeout(self):
+        """Test invalid client login timeout"""
+
+        with self.assertRaises(ValueError):
+            await self.connect(login_timeout=-1)
+
 
 class _TestServerLoginTimeoutExceeded(ServerTestCase):
     """Unit test for server login timeout"""
