@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2015-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -275,8 +275,8 @@ class _TestKex(AsyncTestCase):
     async def test_dh_gex_old(self):
         """Unit test old DH group exchange request"""
 
-        register_kex_alg(b'dh-gex-sha1-1024', _KexDHGex, sha1, 1024)
-        register_kex_alg(b'dh-gex-sha1-2048', _KexDHGex, sha1, 2048)
+        register_kex_alg(b'dh-gex-sha1-1024', _KexDHGex, sha1, (1024,), True)
+        register_kex_alg(b'dh-gex-sha1-2048', _KexDHGex, sha1, (2048,), True)
 
         for size in (b'1024', b'2048'):
             with self.subTest('Old DH group exchange', size=size):
@@ -286,11 +286,16 @@ class _TestKex(AsyncTestCase):
     async def test_dh_gex(self):
         """Unit test old DH group exchange request"""
 
-        register_kex_alg(b'dh-gex-sha1-1024-1536', _KexDHGex, sha1, 1024, 1536)
-        register_kex_alg(b'dh-gex-sha1-1536-3072', _KexDHGex, sha1, 1536, 3072)
-        register_kex_alg(b'dh-gex-sha1-2560-2560', _KexDHGex, sha1, 2560, 2560)
-        register_kex_alg(b'dh-gex-sha1-2560-4096', _KexDHGex, sha1, 2560, 4096)
-        register_kex_alg(b'dh-gex-sha1-9216-9216', _KexDHGex, sha1, 9216, 9216)
+        register_kex_alg(b'dh-gex-sha1-1024-1536', _KexDHGex, sha1,
+                         (1024, 1536), True)
+        register_kex_alg(b'dh-gex-sha1-1536-3072', _KexDHGex, sha1,
+                         (1536, 3072), True)
+        register_kex_alg(b'dh-gex-sha1-2560-2560', _KexDHGex, sha1,
+                         (2560, 2560), True)
+        register_kex_alg(b'dh-gex-sha1-2560-4096', _KexDHGex, sha1,
+                         (2560, 4096), True)
+        register_kex_alg(b'dh-gex-sha1-9216-9216', _KexDHGex, sha1,
+                         (9216, 9216), True)
 
         for size in (b'1024-1536', b'1536-3072', b'2560-2560',
                      b'2560-4096', b'9216-9216'):
