@@ -1,11 +1,19 @@
-# Copyright (c) 2015-2017 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2015-2018 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -201,6 +209,18 @@ class _TestKnownHosts(TempDirTestCase):
         for patlists in no_match:
             self.check_hosts(patlists, ([], [], [], [], [], [], []))
 
+    def test_scoped_addr(self):
+        """Test match on scoped addresses"""
+
+        self.check_hosts((['fe80::1%1'], [], [], [], [], [], []),
+                         ([0], [], [], [], [], [], []), addr='fe80::1%1')
+        self.check_hosts((['fe80::%1/64'], [], [], [], [], [], []),
+                         ([0], [], [], [], [], [], []), addr='fe80::1%1')
+        self.check_hosts((['fe80::1%2'], [], [], [], [], [], []),
+                         ([], [], [], [], [], [], []), addr='fe80::1%1')
+        self.check_hosts((['2001:2::%3/64'], [], [], [], [], [], []),
+                         ([0], [], [], [], [], [], []), addr='2001:2::1')
+
     def test_missing_key(self):
         """Test for line with missing key data"""
 
