@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2014-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -41,6 +41,7 @@ import asyncssh
 from asyncssh.asn1 import der_encode, BitString, ObjectIdentifier
 from asyncssh.asn1 import TaggedDERObject
 from asyncssh.crypto import chacha_available, ed25519_available, ed448_available
+from asyncssh.misc import write_file
 from asyncssh.packet import MPInt, String, UInt32
 from asyncssh.pbe import pkcs1_decrypt
 from asyncssh.public_key import CERT_TYPE_USER, CERT_TYPE_HOST, SSHKey
@@ -49,6 +50,7 @@ from asyncssh.public_key import decode_ssh_certificate
 from asyncssh.public_key import get_public_key_algs, get_certificate_algs
 from asyncssh.public_key import get_x509_certificate_algs
 from asyncssh.public_key import import_certificate_subject
+from asyncssh.public_key import load_identities
 
 from .sk_stub import sk_available, stub_sk, unstub_sk
 from .util import bcrypt_available, x509_available
@@ -72,6 +74,9 @@ _openssl_available = _openssl_version != b''
 # The openssl "-v2prf" option is only available in OpenSSL 1.0.2 or later
 _openssl_supports_v2prf = _openssl_version >= b'OpenSSL 1.0.2'
 
+# Ed25519/Ed448 support via "pkey" is only available in OpenSSL 1.1.1 or later
+_openssl_supports_pkey = _openssl_version >= b'OpenSSL 1.1.1'
+
 try:
     if sys.platform != 'win32':
         _openssh_version = run('ssh -V')
@@ -282,11 +287,21 @@ class _TestPublicKey(TempDirTestCase):
             with self.assertRaises((asyncssh.KeyEncryptionError,
                                     asyncssh.KeyImportError)):
                 asyncssh.load_keypairs('new', 'xxx')
+
+            if format_name == 'openssh':
+                identities = load_identities(['new'])
+                self.assertEqual(identities[0], pubdata)
+            else:
+                with self.assertRaises(asyncssh.KeyImportError):
+                    load_identities(['new'])
+
+                identities = load_identities(['new'], skip_private=True)
+                self.assertEqual(identities, [])
         else:
             newkey.write_private_key('list', format_name)
             newkey.append_private_key('list', format_name)
 
-            keylist = asyncssh.load_keypairs('list')
+            keylist = asyncssh.read_private_key_list('list')
             self.assertEqual(keylist[0].public_data, pubdata)
             self.assertEqual(keylist[1].public_data, pubdata)
 
@@ -307,6 +322,10 @@ class _TestPublicKey(TempDirTestCase):
             self.assertEqual(keypair.public_data, chain.public_data)
             self.assertIsNotNone(keypair.get_agent_private_key())
 
+            keypair = asyncssh.load_keypairs('new', passphrase, 'new_cert')[0]
+            self.assertEqual(keypair.public_data, chain.public_data)
+            self.assertIsNotNone(keypair.get_agent_private_key())
+
             newkey.write_private_key('new_bundle', format_name, passphrase)
             cert.append_certificate('new_bundle', 'pem')
 
@@ -320,41 +339,44 @@ class _TestPublicKey(TempDirTestCase):
         """Check for a public key match"""
 
         newkey = asyncssh.read_public_key('new')
-        pubdata = newkey.export_public_key()
+        pubkey = newkey.export_public_key()
+        pubdata = newkey.public_data
 
         self.assertEqual(newkey, self.pubkey)
         self.assertEqual(hash(newkey), hash(self.pubkey))
 
-        keypair = asyncssh.load_public_keys('new')[0]
-        self.assertEqual(keypair, newkey)
+        pubkey = asyncssh.load_public_keys('new')[0]
+        self.assertEqual(pubkey, newkey)
 
-        keypair = asyncssh.load_public_keys([newkey])[0]
-        self.assertEqual(keypair, newkey)
+        pubkey = asyncssh.load_public_keys([newkey])[0]
+        self.assertEqual(pubkey, newkey)
 
-        keypair = asyncssh.load_public_keys([pubdata])[0]
-        self.assertEqual(keypair, newkey)
+        pubkey = asyncssh.load_public_keys([pubkey])[0]
+        self.assertEqual(pubkey, newkey)
 
-        keypair = asyncssh.load_public_keys(['new'])[0]
-        self.assertEqual(keypair, newkey)
+        pubkey = asyncssh.load_public_keys(['new'])[0]
+        self.assertEqual(pubkey, newkey)
 
-        keypair = asyncssh.load_public_keys(Path('new'))[0]
-        self.assertEqual(keypair, newkey)
+        pubkey = asyncssh.load_public_keys(Path('new'))[0]
+        self.assertEqual(pubkey, newkey)
 
-        keypair = asyncssh.load_public_keys([Path('new')])[0]
-        self.assertEqual(keypair, newkey)
+        pubkey = asyncssh.load_public_keys([Path('new')])[0]
+        self.assertEqual(pubkey, newkey)
+
+        identity = load_identities(['new'])[0]
+        self.assertEqual(identity, pubdata)
 
         newkey.write_public_key('list', format_name)
         newkey.append_public_key('list', format_name)
 
-        keylist = asyncssh.load_public_keys('list')
+        keylist = asyncssh.read_public_key_list('list')
         self.assertEqual(keylist[0], newkey)
         self.assertEqual(keylist[1], newkey)
 
         newkey.write_public_key(Path('list'), format_name)
         newkey.append_public_key(Path('list'), format_name)
 
-        with open('list', 'ab') as f:
-            f.write(b'Extra text at end of key list\n')
+        write_file('list', b'Extra text at end of key list\n', 'ab')
 
         keylist = asyncssh.load_public_keys(Path('list'))
         self.assertEqual(keylist[0], newkey)
@@ -425,8 +447,7 @@ class _TestPublicKey(TempDirTestCase):
         cert.write_certificate(Path('list'), format_name)
         cert.append_certificate(Path('list'), format_name)
 
-        with open('list', 'ab') as f:
-            f.write(b'Extra text at end of certificate list\n')
+        write_file('list', b'Extra text at end of certificate list\n', 'ab')
 
         certlist = asyncssh.load_certificates(Path('list'))
         self.assertEqual(certlist[0], cert)
@@ -583,9 +604,13 @@ class _TestPublicKey(TempDirTestCase):
 
         format_name = 'pkcs8-%s' % fmt
 
-        if self.use_openssl: # pragma: no branch
-            run('openssl %s -pubin -in pub -inform pem -out new -outform %s' %
-                (self.keyclass, fmt))
+        if self.use_openssl:
+            if _openssl_supports_pkey:
+                run('openssl pkey -pubin -in pub -inform pem -out new '
+                    '-outform %s' % fmt)
+            else: # pragma: no cover
+                run('openssl %s -pubin -in pub -inform pem -out new '
+                    '-outform %s' % (self.keyclass, fmt))
         else: # pragma: no cover
             self.pubkey.write_public_key('new', format_name)
 
@@ -597,9 +622,13 @@ class _TestPublicKey(TempDirTestCase):
         format_name = 'pkcs8-%s' % fmt
         self.privkey.write_public_key('pubout', format_name)
 
-        if self.use_openssl: # pragma: no branch
-            run('openssl %s -pubin -in pubout -inform %s -out new '
-                '-outform pem' % (self.keyclass, fmt))
+        if self.use_openssl:
+            if _openssl_supports_pkey:
+                run('openssl pkey -pubin -in pubout -inform %s -out new '
+                    '-outform pem' % fmt)
+            else: # pragma: no cover
+                run('openssl %s -pubin -in pubout -inform %s -out new '
+                    '-outform pem' % (self.keyclass, fmt))
         else: # pragma: no cover
             pub = asyncssh.read_public_key('pubout')
             pub.write_public_key('new', format_name)
@@ -695,9 +724,7 @@ class _TestPublicKey(TempDirTestCase):
         self.check_public('rfc4716')
 
         pubdata = self.pubkey.export_public_key('rfc4716')
-
-        with open('new', 'wb') as f:
-            f.write(pubdata.replace(b'\n', b'\nXXX:\n', 1))
+        write_file('new', pubdata.replace(b'\n', b'\nXXX:\n', 1))
 
         self.check_public('rfc4716')
 
@@ -1135,6 +1162,20 @@ class _TestPublicKey(TempDirTestCase):
              b'ssh-dss ' + binascii.b2a_base64(String('xxx'))),
             ('Invalid OpenSSH body',
              b'ssh-dss ' + binascii.b2a_base64(String('ssh-dss'))),
+            ('Unknown format OpenSSH key',
+             b'-----BEGIN OPENSSH PRIVATE KEY-----\n' +
+             binascii.b2a_base64(b'XXX') +
+             b'-----END OPENSSH PRIVATE KEY-----'),
+            ('Incomplete OpenSSH key',
+             b'-----BEGIN OPENSSH PRIVATE KEY-----\n' +
+             binascii.b2a_base64(b'openssh-key-v1\0') +
+             b'-----END OPENSSH PRIVATE KEY-----'),
+            ('Invalid OpenSSH nkeys',
+             b'-----BEGIN OPENSSH PRIVATE KEY-----\n' +
+             binascii.b2a_base64(b''.join(
+                 (b'openssh-key-v1\0', String(''), String(''), String(''),
+                  UInt32(2), String(''), String('')))) +
+             b'-----END OPENSSH PRIVATE KEY-----'),
             ('Invalid RFC4716 header', b'---- XXX ----\n'),
             ('Missing RFC4716 footer', b'---- BEGIN SSH2 PUBLIC KEY ----\n'),
             ('Invalid RFC4716 header',
@@ -1236,7 +1277,7 @@ class _TestPublicKey(TempDirTestCase):
 
             with self.subTest('Sign with bad algorithm'):
                 with self.assertRaises(ValueError):
-                    self.privkey.sign(data, 'xxx')
+                    self.privkey.sign(data, b'xxx')
 
             with self.subTest('Verify with bad algorithm'):
                 self.assertFalse(self.pubkey.verify(
@@ -1423,6 +1464,9 @@ class _TestPublicKey(TempDirTestCase):
             keypair = asyncssh.load_keypairs((priv, self.usercert))[0]
             self.assertEqual(keypair.get_comment(), 'user_comment')
 
+            keypair = asyncssh.load_keypairs(priv, None, self.usercert)[0]
+            self.assertEqual(keypair.get_comment(), 'user_comment')
+
             pubdata = self.pubkey.export_public_key()
             keypair = asyncssh.load_keypairs((priv, pubdata))[0]
             self.assertEqual(keypair.get_comment(), 'pub_comment')
@@ -1431,6 +1475,9 @@ class _TestPublicKey(TempDirTestCase):
             keypair = asyncssh.load_keypairs((priv, certdata))[0]
             self.assertEqual(keypair.get_comment(), 'user_comment')
 
+            keypair = asyncssh.load_keypairs(priv, None, certdata)[0]
+            self.assertEqual(keypair.get_comment(), 'user_comment')
+
             priv.write_private_key('key')
 
             keypair = asyncssh.load_keypairs('key')[0]
@@ -1442,6 +1489,9 @@ class _TestPublicKey(TempDirTestCase):
             keypair = asyncssh.load_keypairs(('key', 'usercert'))[0]
             self.assertEqual(keypair.get_comment(), 'user_comment')
 
+            keypair = asyncssh.load_keypairs('key', None, 'usercert')[0]
+            self.assertEqual(keypair.get_comment(), 'user_comment')
+
             self.pubkey.write_public_key('key.pub')
 
             keypair = asyncssh.load_keypairs('key')[0]
@@ -1829,9 +1879,7 @@ class _TestPublicKey(TempDirTestCase):
 
         with self.subTest('Invalid DER format in certificate list'):
             with self.assertRaises(asyncssh.KeyImportError):
-                with open('certlist', 'wb') as f:
-                    f.write(b'\x30\x00')
-
+                write_file('certlist', b'\x30\x00')
                 asyncssh.read_certificate_list('certlist')
 
         with self.subTest('Invalid PEM format'):
@@ -2069,12 +2117,14 @@ class TestEd25519(_TestPublicKey):
     """Test Ed25519 keys"""
 
     keyclass = 'ed25519'
-    base_format = 'openssh'
+    base_format = 'pkcs8-pem'
     private_formats = ('pkcs8', 'openssh')
     public_formats = ('pkcs8', 'openssh', 'rfc4716')
+    x509_supported = x509_available
     default_cert_version = 'ssh-ed25519-cert-v01@openssh.com'
     generate_args = (('ssh-ed25519', {}),)
-    use_openssl = False
+    use_openssh = False
+    use_openssl = _openssl_supports_pkey
 
 
 @unittest.skipUnless(ed448_available, 'ed448 not available')
@@ -2082,13 +2132,14 @@ class TestEd448(_TestPublicKey):
     """Test Ed448 keys"""
 
     keyclass = 'ed448'
-    base_format = 'openssh'
+    base_format = 'pkcs8-pem'
     private_formats = ('pkcs8', 'openssh')
     public_formats = ('pkcs8', 'openssh', 'rfc4716')
+    x509_supported = x509_available
     default_cert_version = 'ssh-ed448-cert-v01@openssh.com'
     generate_args = (('ssh-ed448', {}),)
     use_openssh = False
-    use_openssl = False
+    use_openssl = _openssl_supports_pkey
 
 
 @unittest.skipUnless(sk_available, 'security key support not available')
@@ -2128,7 +2179,6 @@ class TestSKEd25519(_TestPublicKey):
     default_cert_version = 'sk-ssh-ed25519-cert-v01@openssh.com'
     generate_args = (('sk-ssh-ed25519@openssh.com', {}),)
     use_openssh = False
-    use_openssl = False
 
     def setUp(self):
         """Set up Ed25519 security key test"""
@@ -2160,8 +2210,7 @@ class _TestPublicKeyTopLevel(TempDirTestCase):
         with self.assertRaises(asyncssh.KeyImportError):
             asyncssh.import_public_key(keydata)
 
-        with open('list', 'wb') as f:
-            f.write(keydata)
+        write_file('list', keydata)
 
         with self.assertRaises(asyncssh.KeyImportError):
             asyncssh.read_public_key_list('list')
@@ -2257,3 +2306,19 @@ class _TestPublicKeyTopLevel(TempDirTestCase):
         privkey = asyncssh.generate_private_key('ssh-rsa', 2048)
 
         self.assertIsNone(privkey.decrypt(b'', privkey.algorithm))
+
+    @unittest.skipUnless(x509_available, 'x509 not available')
+    def test_x509_certificate_hashes(self):
+        """Test X.509 certificate hash algorithms"""
+
+        privkey = asyncssh.generate_private_key('ssh-rsa')
+        pubkey = privkey.convert_to_public()
+
+        for hash_alg in ('sha1', 'sha256', 'sha512'):
+            cert = privkey.generate_x509_user_certificate(
+                pubkey, 'OU=user', hash_alg=hash_alg)
+
+            cert.write_certificate('cert', 'pem')
+
+            cert2 = asyncssh.read_certificate('cert')
+            self.assertEqual(str(cert2.subject), 'OU=user')
